
'''
    Created on Jun 21, 2012
    
    @author: ksahlin
    
    This file is part of BESST.
    
    BESST is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    BESST is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with BESST.  If not, see <http://www.gnu.org/licenses/>.
    '''

import time
import networkx as nx
from collections import defaultdict
import multiprocessing as mp


def ScorePaths(G, paths, all_paths, param):
    if len(paths) == 0:
        return ()

    def CalculateConnectivity(path, G):
        good_edges = set(map(lambda i: path[i], filter(lambda i: i % 2 == 1, range(len(path)))))
        #print good_edges, 'path len:',len(path)- 2, 'nr of good edges: ', len(good_edges)*(len(good_edges)+1)/2.0
        good_edges_count = 0
        bad_edges_count = 0
        prev_node = path[0]
        good_edges_already_considered = set()
        bad_edges_already_considered = set()
        bad_link_weight = 0
        good_link_weight = 0
        link_weights = defaultdict(lambda : defaultdict(float))

        for node in path:
            if node[0] == prev_node[0]:
                #TODO: eventually calculate if global optima here. We then need a hash table good_e ={node:[tot_edge = set(nbr1,nbr2..), bad_edge = set(nbr3,..)]}
                for nbr in G.neighbors(node):
                    if nbr in good_edges and not nbr[0] == node[0]:
                        good_edges_count += 1
                        good_link_weight += G[node][nbr]['nr_links']
                        link_weights[node[0]]['good'] += G[node][nbr]['nr_links']
                        good_edges_already_considered.add((nbr, node))
                        good_edges_already_considered.add((node, nbr))
                    elif not nbr[0] == node[0] and (nbr, node) not in bad_edges_already_considered:
                        bad_edges_count += 1
                        bad_link_weight += G[node][nbr]['nr_links']
                        link_weights[node[0]]['bad'] += G[node][nbr]['nr_links']
                        bad_edges_already_considered.add((nbr, node))
                        bad_edges_already_considered.add((node, nbr))
            else:
                good_edges.remove(node)
                for nbr in G.neighbors(node):
                    if (nbr, node) not in good_edges_already_considered and not nbr[0] == node[0] and (nbr, node) not in bad_edges_already_considered:
                        bad_edges_count += 1
                        bad_link_weight += G[node][nbr]['nr_links']
                        link_weights[node[0]]['bad'] += G[node][nbr]['nr_links']
                        bad_edges_already_considered.add((nbr, node))
                        bad_edges_already_considered.add((node, nbr))
                    elif (nbr, node) in good_edges_already_considered:
                        link_weights[node[0]]['good'] += G[node][nbr]['nr_links']
            prev_node = node


        try:
            score = good_link_weight / float(bad_link_weight)
        except ZeroDivisionError:
            score = good_link_weight

        return score, bad_link_weight



    #print '\nSTARTING scoring paths:'
    for path_ in paths:
        path = path_[0]
        path_len = path_[1]
        #calculate spanning score s_ci
        score, bad_link_weight = CalculateConnectivity(path, G)
        if param.no_score and score >= param.score_cutoff:
            all_paths.append([score, bad_link_weight, path, path_len])
            #Insert_path(all_paths, score, path , bad_link_weight, path_len)
        elif len(path) > 2 and score >= param.score_cutoff: #startnode and end node are not directly connected
            all_paths.append([score, bad_link_weight, path, path_len])
            #Insert_path(all_paths, score, path , bad_link_weight, path_len)

    return ()

def find_all_paths_for_start_node(graph, start, end, already_visited, is_withing_scaf, max_path_length_allowed, param):
    path = []
    paths = []
    if start[1] == 'L':
        forbidden = set()
        forbidden.add((start[0], 'R'))
    else:
        forbidden = set()
        forbidden.add((start[0], 'L'))

    #Joining within scaffolds
    if is_withing_scaf:
        element = end.pop()
        end.add(element)
        if element[1] == 'L':
            forbidden.add((element[0], 'R'))
        else:
            forbidden.add((element[0], 'L'))


    #TODO: Have length criteria that limits the path lenght due to complecity reasons. Can also identify strange
    #links by looking how many neighbors a contig has and how mych the library actually can span
    path_len = 0
    queue = [(start, path, path_len)]#, sum_path)]
    #prev_node = start
    counter = 0
    while queue:
        #prev_node = start
        counter += 1
        #if counter % 100 == 0:
        #    print 'Potential paths:', counter, 'paths found: ', len(paths)
        if counter > param.path_threshold or len(path) > 100:
            break
            
        start, path, path_len = queue.pop() #start, end, path, sum_path = queue.pop()  
        try:
            prev_node = path[-1]
        except IndexError:
            prev_node = start
        path = path + [start]
        path_len = len(path)
        #print 'PATH', path ,'end', end 
        if path_len > max_path_length_allowed: #All possible paths can be exponential!! need something to stop algorithm in time
            continue
        #if score < score_best_path: # need something to stop a bad path
        #    continue
        if start in already_visited or start in forbidden:
            continue

        if start in end:
            # if (start_node, start) in nodes_present_in_path:
            #     nodes_present_in_path[(start_node, start)] = nodes_present_in_path[(start_node, start)].union(path)
            # else:
            #     nodes_present_in_path[(start_node, start)] = set(path)
            paths.append((path, path_len))
            continue


        if  prev_node[0] != start[0]:
            if start[1] == 'L' and (start[0], 'R') not in forbidden:
                queue.append(((start[0], 'R'), path, path_len)) #, sum_path + graph[start][(start[0], 'R')]['nr_links']))
            elif start[1] == 'R' and (start[0], 'L') not in forbidden:
                queue.append(((start[0], 'L'), path, path_len))#, sum_path + graph[start][(start[0], 'L')]['nr_links']))                
        else:
            for node in set(graph[start]).difference(path):
                if node not in forbidden: # and node not in already_visited: 
                    try: # if last node (i.e. "end") it is not present in small_scaffolds and it should not be included in the length
                        queue.append((node, path, path_len + graph[node[0]]['length'])) #  small_scaffolds[node[0]].s_length))   #
                    except KeyError:
                        queue.append((node, path, path_len))

    return paths



def BetweenScaffolds(G_prime, end, iter_nodes, param):
    # here we should have a for loop looping over all start nodes. Start nodes already examined should be removed in a nice way to skip over counting
    already_visited = set()
    all_paths = []
    print 'Entering "find_all_paths_for_start_node" '
    iter_count = 0
    cnter = 0
    if param.max_extensions:
        iter_threshold = param.max_extensions
    else: 
        iter_threshold = len(end)

    print 'iterating until maximum of {0} extensions.'.format(iter_threshold) 
    print 'nodes:{0}, edges: {1}'.format(len(G_prime.nodes()), len(G_prime.edges()))
    while len(iter_nodes) > 0 and iter_count <= iter_threshold:
        iter_count += 1
        start_node = iter_nodes.pop()
        if cnter % 100 == 0:
            print 'enter Betwween scaf node: ', cnter
        end.difference_update(set([start_node]))
        paths = find_all_paths_for_start_node(G_prime, start_node, end, already_visited, 0, 2 ** 32, param)
        already_visited.add(start_node)
        ScorePaths(G_prime, paths, all_paths, param)
        cnter += 1
    #all_paths = ExtendScaffolds(all_paths)
    #print all_paths
    print 'Total nr of paths found: {0} with score larger than: {1}'.format(len(all_paths), param.score_cutoff)
    all_paths.sort(key=lambda list_: list_[0]) 
    #print all_paths
    return(all_paths)

def WithinScaffolds(G, G_prime, start, end_node, already_visited, max_path_length, param):
    end = set()
    end.add(end_node)
    all_paths = []
    already_visited.difference_update(set([start, end_node]))
    paths = find_all_paths_for_start_node(G_prime, start, end, already_visited, 1, max_path_length, param)
    already_visited.add(start)
    already_visited.add(end_node)
    #print paths
    if len(paths) > 1:
        ScorePaths(G_prime, paths, all_paths,param)
        all_paths.sort(key=lambda list_: list_[0]) 

        if len(all_paths) > 0:
            return all_paths

    return []

if __name__ == '__main__':
    import Scaffold
    small_scaffolds_test = {}
    for i in range(1, 7):
        S = Scaffold.scaffold(i, 0, 0, {}, {})
        small_scaffolds_test[S.name] = S
    start = time()
    G_prime = nx.Graph()
    #G.add_nodes_from([(1, 'L'), (1, 'R'), (2, 'L'), (2, 'R'), (3, 'L'), (3, 'R'), (4, 'L'), (4, 'R'), (5, 'L'), (5, 'R')]) 
    for i in range(1, 7):
        G_prime.add_edge((i, 'L'), (i, 'R'), {'nr_links':0})
    G_prime.add_edges_from([((1, 'R'), (2, 'R'), {'nr_links':1}), ((3, 'L'), (4, 'L'), {'nr_links':1}), ((2, 'L'), (3, 'R'), {'nr_links':1}), ((1, 'R'), (5, 'L'), {'nr_links':2}),
                       ((5, 'R'), (4, 'L'), {'nr_links':3}), ((2, 'L'), (5, 'L'), {'nr_links':2}), ((1, 'R'), (4, 'L'), {'nr_links':8}), ((2, 'L'), (6, 'L'), {'nr_links':3}),
                       ((1, 'L'), (4, 'R'), {'nr_links':1}), ((1, 'L'), (4, 'L'), {'nr_links':1}), ((3, 'L'), (4, 'R'), {'nr_links':1}),
                        ((1, 'R'), (2, 'L'), {'nr_links':1}), ((1, 'R'), (5, 'R'), {'nr_links':1}), ((2, 'L'), (5, 'R'), {'nr_links':1})])
    G = nx.Graph()
    G.add_nodes_from([(1, 'L'), (1, 'R'), (4, 'L'), (4, 'R'), (6, 'L'), (6, 'R')])
    contigs = [1, 2, 3, 4, 5, 6]

    print 'Between'
    BetweenScaffolds(G, G_prime, small_scaffolds_test)
    start_node = (1, 'R')
    end_node = (4, 'L')
    print 'Within'
    already_visited = set(G.nodes())
    print already_visited
    WithinScaffolds(G, G_prime, small_scaffolds_test, start_node, end_node, already_visited, 0)
    elapsed = time() - start
    print 'time all paths: ', elapsed





# Translated from Octave code at:
# http://www.ecs.shimane-u.ac.jp/~kyoshida/lpeng.htm
# and placed under MIT licence by Enzo Michelangeli with permission explicitly
# granted by the original author, Prof. Kazunobu Yoshida
#
# -----------------------------------------------------------------------------
# Copyright (c) 2010, Kazunobu Yoshida, Shimane University, and Enzo Michelangeli,
# IT Vision Limited
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

from numpy import *

class Result(dict):
    """ Represents the optimization result.

    Attributes
    ----------
    x : ndarray
        The solution of the optimization.
    success : bool
        Whether or not the optimizer exited successfully.
    status : int
        Termination status of the optimizer. Its value depends on the
        underlying solver. Refer to `message` for details.
    message : str
        Description of the cause of the termination.
    fun, jac, hess : ndarray
        Values of objective function, Jacobian and Hessian (if available).
    nfev, njev, nhev: int
        Number of evaluations of the objective functions and of its
        Jacobian and Hessian.
    nit: int
        Number of iterations performed by the optimizer.

    Notes
    -----
    There may be additional attributes not listed above depending of the
    specific solver. Since this class is essentially a subclass of dict
    with attribute accessors, one can see which attributes are available
    using the `keys()` method.
    """
    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __repr__(self):
        if self.keys():
            m = max(map(len, self.keys())) + 1
            return '\n'.join([k.rjust(m) + ': ' + repr(v)
                              for k, v in self.iteritems()])
        else:
            return self.__class__.__name__ + "()"


class LPResult(Result):
    """
    Solution to a linear programming problem

    Attributes
    ----------
    x : ndarray
        The solution of the optimization.
    success : bool
        Whether or not the optimizer exited successfully.
    fun : float
        The optimal value
    is_bounded : bool
        True if the solution is bounded; False if unbounded
    is_solvable : bool
        True if the problem is solvable; False if unsolvable
    basis : ndarray
        Indices of the basis of the solution.
    """
    _fields = ['x', 'min', 'is_bounded', 'solvable', 'basis']

def lp_solve(c, A, b, tol=1e-10):
    """
    Solves a linear programming problem using a two-phase method.

    The problem solved is::

        minimize    z = c' x
        subject to  A x = b, x >= 0

    Parameters
    ----------
    c : array_like
    A : array_like
    b : array_like
        Problem parameters
    tol : float, optional
        Tolerance

    Returns
    -------
    sol : LPResult
        A solution object, with the properties:
            x : ndarray
                An optimal solution.
            fun : float
                The optimal value.
            is_bounded : bool
                True if the solution is bounded; False if unbounded.
            is_solvable : bool
                True if the problem is solvable; False if unsolvable.
            basis : ndarray
                Indices of the basis of the solution.

    Notes
    -----
    The solution is found using the two phase method, where the
    simplex method is used at each stage.

    """
    c = asarray(c)
    A = asarray(A)
    b = asarray(b)

    m,n = A.shape   # m: number of constraints; n: number of variables
    for i in xrange(m):
        if b[i] < 0.0:
            A[i,:] = -A[i,:]
            b[i] = -b[i]
    d = -sum(A, axis=0)
    w0 = sum(b)
    H = vstack([     #  The initial simplex table of phase one
         hstack([A, array([b]).T]), # first m rows
         hstack([c, 0.]),   # last-but-one
         hstack([d, -w0])]) # last
    indx = range(n)
    basis = arange(n, n+m) # m elements from n to n+m-1
    is_bounded = _simplex(H, basis, indx, 1)
    if H[m+1,n] < -tol:   # last row, last column
        sol = False
        optx = None
        zmin = None
        is_bounded = None
    else:
        sol = True
        j = -1
        for i in xrange(n):
            j = j+1
            if H[m+1,j] > tol:
                H = delete(H, j, 1)     # H[:,j] = [] # delete column j from H
                del indx[j]
                j = j-1
        H = delete(H, m+1, 0)
        if indx > 0:
            # Phase two
            is_bounded = _simplex(H,basis,indx,2)
            if is_bounded:
                optx = zeros(n+m)
                for i in xrange(m):
                    optx[basis[i]] = H[i,-1]
                optx = optx[0:n]
                zmin = -H[-1,-1]    #  last row, last column
            else:
                optx = None
                zmin = -Inf
        else:
            optx = zeros(n)
            zmin = 0

    return LPResult(x=optx,
                    fun=zmin,
                    success=(sol and is_bounded),
                    is_bounded=is_bounded,
                    is_solvable=sol,
                    basis=basis)

def _simplex(H,basis,indx,s):
    '''
      [H1,basis,is_bounded] = _simplex(H,basis,indx,s)
      H: simplex table (MODIFIED).
      basis: the indices of basis (MODIFIED).
      indx: the indices of x.
      s: 1 for phase one; 2 for phase two.
      H1: new simplex table.
      is_bounded: True if the solution is bounded; False if unbounded.
    '''
    if s == 1:
        s0 = 2
    elif s == 2:
        s0 = 1
    n1 = H.shape[0]
    sol = False
    while not sol:
        q = H[-1, :-1] # last row, all columns but last
        jp = argmin(q)
        fm = q[jp]
        if fm >= 0:
            is_bounded = True    # bounded solution
            sol = True
        else:
            q = H[:-s0,jp]
            ip = argmax(q)
            hm = q[ip]
            if hm <= 0:
                is_bounded = False # unbounded solution
                sol = True
            else:
                h1 = zeros(n1-s0)
                for i in xrange(n1-s0):
                    if H[i,jp] > 0:
                        h1[i] = H[i,-1]/H[i,jp]
                    else:
                        h1[i] = Inf
                ip = argmin(h1)
                minh1 = h1[ip]
                basis[ip] = indx[jp]
                if not _pivot(H,ip,jp):
                    raise ValueError("the first parameter is a Singular matrix")
    return is_bounded

def _pivot(H,ip,jp):
    # H is MODIFIED
    n, m = H.shape
    piv = H[ip,jp]
    if piv == 0:
        #print('singular')
        return False
    else:
        H[ip,:] /= piv
        for i in xrange(n):
            if i != ip:
                H[i,:] -= H[i,jp]*H[ip,:]
    return True


import os
import sys
import pysam

##
# Opens a .bam or .sam file and returns the file
# object.
#
# @param bam_file_path Path to the .bam or .sam.
# 
# @return File object for .bam or .sam.
#
def open_bam_file(bam_file_path):
    bam_file_name, bam_file_ext = os.path.splitext(bam_file_path)
    if bam_file_ext == ".bam":
        return pysam.Samfile(bam_file_path, 'rb')
    elif bam_file_ext == ".sam":
        return pysam.Samfile(bam_file_path, 'r')
    else:
        return IOError("open_bam_file: File must be either .bam or .sam.")

def is_proper_aligned_unique_innie(read):
    return (read.is_reverse and not read.mate_is_reverse and read.is_read2 and read.tlen < 0 and read.rname == read.mrnm) or \
                (not read.is_reverse and read.mate_is_reverse and read.is_read2 and read.tlen > 0 and read.rname == read.mrnm ) \
                and not read.mate_is_unmapped and read.mapq > 10 and not read.is_secondary
def is_proper_aligned_unique_outie(read):
    return (read.is_reverse and not read.mate_is_reverse and read.is_read2 and read.tlen > 0 and read.rname == read.mrnm) or \
                (not read.is_reverse and read.mate_is_reverse and read.is_read2 and read.tlen < 0 and read.rname == read.mrnm ) \
                and not read.mate_is_unmapped and read.mapq > 10 and not read.is_secondary

def is_unique_read_link(read):
    # if  not read.is_unmapped and not read.mate_is_unmapped and read.rname != read.mrnm \
    # and read.opt('XT')=='U' and not read.is_secondary and read.rlen != read.alen:
    #     print read
    return not read.is_unmapped and not read.mate_is_unmapped and read.rname != read.mrnm \
    and read.mapq > 10 and not read.is_secondary


### BOWTIE ####
def proper_unique_alignment_innie_bowtie(read1,read2):
    unique1 = not read1.is_unmapped and not read1.is_secondary 
    unique2 = not read2.is_unmapped and not read2.is_secondary 
    is_innie = not read1.is_reverse and read2.is_reverse and read1.pos < read2.pos \
                or read1.is_reverse and not read2.is_reverse and read1.pos > read2.pos
    return unique1 and unique2 and is_innie 

def proper_unique_alignment_outie_bowtie(read1,read2):
    unique1 = not read1.is_unmapped and not read1.is_secondary 
    unique2 = not read2.is_unmapped and not read2.is_secondary  
    is_outie = read1.is_reverse and not read2.is_reverse and read1.pos < read2.pos \
                or not read1.is_reverse and read2.is_reverse and read1.pos > read2.pos
    return unique1 and unique2 and is_outie 

def unique_link_alignment_bowtie(read1,read2):
    unique1 = not read1.is_unmapped and not read1.is_secondary 
    unique2 = not read2.is_unmapped and not read2.is_secondary  
    return unique1 and unique2 

##############

def get_mp_observation(read1, read2, ctg_len1, ctg_len2):
    if not read1.is_reverse:
        obs1 = read1.pos + read1.qlen
    else:
        obs1 = ctg_len1 - read1.pos

    if not read2.is_reverse:
        obs2 = read2.pos + read2.qlen
    else:
        obs2 = ctg_len2 - read2.pos
    return obs1, obs2

def get_pe_observation(read1, read2, ctg_len1, ctg_len2):
    if read1.is_reverse:
        obs1 = read1.pos + read1.qlen
    else:
        obs1 = ctg_len1 - read1.pos

    if read2.is_reverse:
        obs2 = read2.pos + read2.qlen
    else:
        obs2 = ctg_len2 - read2.pos
    return obs1, obs2

class BamParser(object):
    """docstring for BamParser"""
    def __init__(self, bam_file,bam_path2 = None):
        super(BamParser, self).__init__()
        self.bam_file = open_bam_file(bam_file)
        if bam_path2:
            self.bam_file2 = open_bam_file(bam_path2) 
        self.contig_lengths = dict(zip(self.bam_file.references,self.bam_file.lengths))

    def proper_aligned_unique_pairs(self,aligner, samples=2**32):
        nr_samples = 0
        if aligner == 'bwa' or aligner == 'bwa_mem':
            for ref, length in sorted(zip(self.bam_file.references,self.bam_file.lengths),key = lambda x: x[1], reverse = True):
                try:
                    iter_ = self.bam_file.fetch(ref)
                except ValueError:
                    sys.stderr.write('Need indexed bamfiles, index file should be located in the same directory as the BAM file\nterminating..\n')
                    sys.exit(0)
                for read in iter_:
                    if read.is_read2 and is_proper_aligned_unique_innie(read):
                        nr_samples += 1
                        yield 'innie',read
                    elif read.is_read2 and is_proper_aligned_unique_outie(read): 
                        nr_samples += 1
                        yield 'outie',read

                    if nr_samples >= samples:
                        break
            self.bam_file.seek(0)


        elif aligner == 'bowtie':
            ctgs_largest_first = [ref for ref, length in  sorted(zip(self.bam_file.references,self.bam_file.lengths), key = lambda x: x[1], reverse = True)]
            long_ctgs = set(ctgs_largest_first[:10])
            for read1,read2 in zip(self.bam_file,self.bam_file2):
                assert read1.qname == read2.qname
                same_ref = read1.rname == read2.rname
                if same_ref and proper_unique_alignment_innie_bowtie(read1,read2): 
                    if not self.bam_file.getrname( read1.tid) in long_ctgs and  not self.bam_file2.getrname(read2.tid) in long_ctgs:
                        continue
                    nr_samples += 1
                    read1.tlen = abs(read1.pos - read2.pos) + read1.rlen
                    yield 'innie',read1
                elif same_ref and proper_unique_alignment_outie_bowtie(read1,read2): 
                    if not self.bam_file.getrname( read1.tid) in long_ctgs and not self.bam_file2.getrname(read2.tid) in long_ctgs:
                        continue
                    nr_samples += 1
                    read1.tlen = abs(read1.pos - read2.pos) + 2*read1.rlen
                    yield 'outie',read1

                if nr_samples >= samples:
                    break
            self.bam_file.seek(0)      

    def aligned_reads(self,aligner):
        if aligner == 'bwa' or aligner == 'bwa_mem':
            for read in self.bam_file:
                if not read.is_unmapped:
                    yield read 
            self.bam_file.seek(0)
        elif aligner == 'bowtie':
            for read1,read2 in zip(self.bam_file,self.bam_file2):
                if not read1.is_unmapped and not read2.is_unmapped:
                    yield read1,read2           
                elif not read1.is_unmapped:
                    yield read1,False
                elif not read2.is_unmapped:
                    yield False, read2

    def unique_reads_on_different_references(self, aligner):
        read_pairs = {}
        if aligner == 'bwa_mem':
            for read in self.bam_file:
                if is_unique_read_link(read):
                    #tmp for tests:
                    #print read.qname[:-1]
                    #print read_pairs
                    if read.qname in read_pairs:
                        #print 'lol'
                        #read2 = read_pairs[read.qname]
                        #if read.tid == read2.tid:    
                        yield read, read_pairs[read.qname]
                        #else: 
                        #    pass
                        del read_pairs[read.qname]
                    else:
                        read_pairs[read.qname] = read
                    


        elif aligner == 'bwa':
            pass

        elif aligner == 'bowtie':
            for read1,read2 in zip(self.bam_file,self.bam_file2):
                assert read1.qname == read2.qname
                same_ref = read1.rname == read2.rname
                if not same_ref and unique_link_alignment_bowtie(read1,read2): 
                    read1.tlen = abs(read1.pos - read2.pos) + read1.rlen
                    yield read1, read2
                elif not same_ref and unique_link_alignment_bowtie(read1,read2): 
                    read1.tlen = abs(read1.pos - read2.pos) + 2*read1.rlen
                    yield read1, read2

        self.bam_file.seek(0)

    def long_reads_for_coverage(self):
        pass
    def long_reads_scaffold_links(self):
        pass

def main(path):
    for read in BamParser(path).reads_for_coverage():
        if read.qlen != 100:
        #print read.tags
            print read.qlen
        #print read.opt('XT')


if __name__ == '__main__':
    main(sys.argv[1])
